<?php

namespace InSegment\ApiCore\Services;

/**
 * Class Logger
 *
 * @author khartakhai@infuseua.com
 * @package App
 */
final class Logger
{

    /**
     * @var string
     */
    protected $logUID;

    /**
     * @var string
     */
    protected $logPath = '';

    /**
     * @var string
     */
    protected $logFile = 'api-log.json';

    /**
     * @var string
     */
    protected $errorFile = 'api-error.htm';

    /**
     * @var string
     */
    protected $logHistory = 'loghistory.txt';

    /**
     *
     * @var array [[string $date, string $type, array|string $data], ...]
     */
    protected $log = [];

    /**
     * @var string
     */
    protected $error = '';

    /**
     * Creates an instance of Logger.
     * 
     * @param string $logPath    [Optional] Path to log files.
     * @param string $logFile    [Optional] File in JSON format.
     * @param string $errorFile  [Optional] File in HTML format.
     * @return Logger
     */
    public function __construct(string $logPath = '', string $logFile = '', string $errorFile = '')
    {
        $this->setLogPath($logPath);
        $this->setLogFile($logFile);
        $this->setErrorFile($errorFile);
    }

    /**
     * 
     * @param string $path Path to log files.
     * @return \self
     */
    public function setLogPath(string $path): self
    {
        if (!empty($path)) {
            $path = realpath($path);
            
            if (!$path) {
                $path = realpath(app('path.storage') . DIRECTORY_SEPARATOR . 'logs');
            }

            $path .= $path ? DIRECTORY_SEPARATOR : '';

            $this->logPath = $path;
        }
        return $this;
    }

    /**
     * 
     * @param string $file File for log output in json format.
     * @return \self
     */
    public function setLogFile(string $file): self
    {
        if (!empty($file)) {
            $this->logFile = $file;
        }
        return $this;
    }

    /**
     * 
     * @param string $file File for error output.
     * @return \self
     */
    public function setErrorFile(string $file): self
    {
        if (!empty($file)) {
            $this->errorFile = $file;
        }
        return $this;
    }

    /**
     * 
     * @param string $tag
     * @param array|string $data
     * @return \self
     */
    public function push(string $tag, $data): self
    {
        if (is_string($data)) {
            $this->error = $data;
            $data = 'see error file';
        }

        $date = (new \DateTime())->format(DATE_ISO8601);
        $tag = "{$date} > {$tag}";
        $n = 1;

        while (isset($this->log["{$tag} ({$n})"])) {
            $n ++;
        }

        $this->log["{$tag} ({$n})"] = $data;
        return $this;
    }

    /**
     * Flushes log and error in respective files.
     */
    public function close()
    {
        if (!empty($this->log) && $this->logFile) {
            $file = $this->uniqueFilename($this->logFile);
            $data = json_encode($this->log, JSON_PRETTY_PRINT);
            $this->write($file, $data);
        }
        $this->log = [];


        if (!empty($this->error) && $this->errorFile) {
            $file = $this->uniqueFilename($this->errorFile);
            $this->write($file, $this->error);
        }
        $this->error = '';

        $files = array_filter(scandir($this->logPath), function($file)
        {
            return strpos($file, '.api-');
        });
        $files = implode("\n", $files);

        $this->write("{$this->logPath}{$this->logHistory}", $files);
        $this->logUID = null;
    }

    /**
     * 
     * @param string $filename
     * @param string $content
     */
    private function write(string $filename, string &$content)
    {
        file_put_contents($filename, $content);
    }

    /**
     * 
     * @param string $filename
     * @return string Returns "/path/to/YYYYMMDD.NUMBER.filename"
     */
    private function uniqueFilename(string $filename): string
    {
        if (!empty($this->logUID)) {
            return "{$this->logPath}{$this->logUID}.{$filename}";
        }

        $date = (new \DateTime())->format('Ymd');
        $number = 1;

        while (true) {
            $this->logUID = "{$date}.{$number}";
            $file = "{$this->logPath}{$this->logUID}.{$filename}";

            if (!file_exists($file)) {
                return $file;
            }

            $number ++;
        }
    }

}
